-------------------------------------------------------------------------------
-- ElvUI Chat Tweaks By Lockslap (US, Bleeding Hollow)
-- <Borderline Amazing>, http://ba-guild.com
-- Based on functionality provided by Prat and/or Chatter
-------------------------------------------------------------------------------
local Module = ElvUI_ChatTweaks:NewModule("Interrupt Announce", "AceConsole-3.0", "AceEvent-3.0")
local L = ElvUI_ChatTweaks.L
Module.name = L["Interrupt Announce"]

local db, options
local defaults = {
	profile = {
		raidWarning = false,
	}
}

local interruptString = L["%s used on %s's %s."]
local interrupts = {
	2139,	-- counterspell (mage)
	96231,	-- rebuke (paladin)
	1766,	-- kick (rogue)
	57994,	-- wind shear (shaman)
	102060,	-- interrupting shout (warrior)
	6552,	-- pummel (warrior)
	19647,	-- spell lock (warlock)
}

local function InArray(needle, haystack)
	for _, value in ipairs(haystack) do
		if value == needle then
			return true
		end
	end
	return false
end

local function SendInterruptMessage(message)
	if Module.debug then
		self:Print(message)
	end

	if IsInRaid() then	-- in raid
		if db.raidwarning then
			if UnitIsGroupLeader("player") or UnitIsRaidOfficer("player") then
				SendChatMessage(message, "RAID_WARNING")
			else
				SendChatMessage(message, "RAID")
			end
		else
			SendChatMessage(message, "RAID")
		end
	elseif IsInGroup() then -- in party
		SendChatMessage(message, "PARTY")
	else -- neither
		Module:Print(message)
	end
end

function Module:COMBAT_LOG_EVENT_UNFILTERED(_, _, event, _, _, srcName, _, _, _, destName, _, _, spellID, _, _, extraID, ...)
	if Module.debug and event == "SPELL_INTERRUPT" then
		print(srcName, spellID, destName, extraID)
	end
	if event == "SPELL_INTERRUPT" and InArray(spellID, interrupts) and (srcName == UnitName("player") or srcName == UnitName("pet")) then
		SendInterruptMessage(interruptString:format(GetSpellLink(spellID), destName, GetSpellLink(extraID)))
	end
end

function Module:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Module:OnDisable()
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Module:OnInitialize()
	self.db = ElvUI_ChatTweaks.db:RegisterNamespace("InterruptAnnounce", defaults)
	db = self.db.profile
	Module.debug = ElvUI_ChatTweaks.db.profile.debugging
end

function Module:GetOptions()
	if not options then
		options = {
			raidWarning = {
				type = "toggle",
				order = 13,
				name = L["Raid Warning"],
				desc = L["Use the raid warning channel if you're in a raid with the proper permissions."],
				get = function() return db.raidWarning end,
				set = function(_, value) db.raidWarning = value end,
			},
		}
	end
	return options
end

function Module:Info()
	return L["Announces when you interrupt your target in raid/party chat."]
end